#include <stdio.h>
#include "atari.h"

  enum ChipType
  {
	AmNone,
	Am010,
	Am020,
	Am040
  };

  enum ChipState
  {
	CmdRead,
	CmdSecond,
	CmdThird,
	CmdAutoSelect,
	CmdProgram,
	CmdErase1,
	CmdErase2,
	CmdErase,
	CmdLimbo
  };

typedef struct _AmdChip AmdChip;

typedef struct _FlashCart {
  UBYTE     TotalBanks;
  BOOL		bModified;
  BOOL      Active;
  AmdChip	*Rom1, *Rom2;
  AmdChip	*activeChip;
} FlashCart;

extern FlashCart *AtariMaxCart;

extern FlashCart *CreateFlashCart(UBYTE banks);
extern void FreeFlashCart(FlashCart *cart);

extern void FlashCartSave(FlashCart *fc, const char *path);
extern void FlashCartFileRead(FlashCart *cart, FILE *fp);
extern void FlashCartFileWrite(FlashCart *cart, FILE *fp);

extern void FlashCartColdStart(FlashCart *fc);
extern BOOL MapFlashCart(FlashCart *cart);

extern UBYTE FlashCartReadHook(UWORD mem);
extern void FlashCartWriteHook(UWORD mem,UBYTE val);

extern UBYTE FlashCartAreaRead(FlashCart *cart, UWORD mem);
extern int FlashCartAccessD5(FlashCart *fc, UWORD mem);

typedef struct _AmdChip {
  enum ChipState cmdState;
  enum ChipType type;
  UBYTE         TotalBanks;
  UWORD			TotalPages;
  UBYTE         ActiveBank;
  FlashCart		*parent;
  unsigned char *Rom;
} AmdChip;

extern AmdChip *CreateAmdChip(enum ChipType ct, FlashCart *cf, int two_chip);
extern void FreeAmdChip(AmdChip *chip);

extern enum ChipType GetChipType(AmdChip *chip);

extern void AmdChipErase(AmdChip *chip, int sector);

extern BOOL AmdChipRomWrite(AmdChip *chip, UWORD mem,UBYTE val);
extern UBYTE AmdChipRomRead(AmdChip *chip, UWORD mem);

extern BOOL MapAmdChip(AmdChip *chip);

extern void AmdChipFileRead(AmdChip *chip, FILE *fp);
extern void AmdChipFileWrite(AmdChip *chip, FILE *fp);
